import { toPrettyTagname } from "@/components/gallery/TagBadge";
import { Button } from "@/components/ui/button";
import { Card, CardContent, CardHeader, CardTitle } from "@/components/ui/card";
import { Input } from "@/components/ui/input";
import { Popover, PopoverContent, PopoverTrigger } from "@/components/ui/popover";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { useTags } from "@/hook/useTags";
import { useState } from "react";

export function TagsPage() {
    const { data, error, isLoading } = useTags();
    const [searchInput, setSearchInput] = useState<string>("");
    const [orderby, setOrderby] = useState<string>("name");
    const [page, setPage] = useState<number>(1);
    const pageSize = 10;

    if (isLoading) {
        return <div>Loading...</div>
    }
    if (error) {
        return <div>Error: {error.message}</div>
    }

    const filteredTags = data?.filter(tag =>
        tag.name.toLowerCase().includes(searchInput.toLowerCase())
    ).sort((a, b) => {
        if (orderby === "name") {
            return a.name.localeCompare(b.name);
        } else if (orderby === "occurs") {
            return b.occurs - a.occurs;
        }
        return 0;
    });
    const paginatedTags = filteredTags?.slice((page - 1) * pageSize, page * pageSize);
    const totalPages = Math.ceil((filteredTags?.length || 0) / pageSize);

    return (
        <div className="p-4">
            <div className="flex space-x-2">
                <Select value={orderby} onValueChange={setOrderby}>
                    <SelectTrigger className="w-[180px]">
                        <SelectValue placeholder="order by" />
                    </SelectTrigger>
                    <SelectContent>
                        <SelectItem value="name">Name</SelectItem>
                        <SelectItem value="occurs">Occurs</SelectItem>
                    </SelectContent>
                </Select>
                <Input
                    className="mb-4"
                    placeholder="Search tags..."
                    value={searchInput}
                    onChange={(e) => {
                        setSearchInput(e.target.value);
                        setPage(1);
                    }}
                />
            </div>
            <div className="grid grid-cols-1 lg:grid-cols-2 2xl:grid-cols-4 xl:grid-cols-3 gap-2 ">
                {paginatedTags?.map(tag => (
                    <Card key={tag.name} className="mb-4">
                        <CardHeader>
                            <CardTitle className="text-2xl">{toPrettyTagname(tag.name)}({tag.occurs})</CardTitle>
                        </CardHeader>
                        <CardContent>
                            {tag.description}
                        </CardContent>
                    </Card>
                ))}
            </div>
            <div className="flex justify-between mt-4">
                <Button onClick={() => setPage(page - 1)} disabled={page <= 1}>
                    이전
                </Button>
                <Popover>
                    <PopoverTrigger asChild>
                        <span>{page} / {totalPages}</span>
                    </PopoverTrigger>
                    <PopoverContent>
                        <Input type="number" value={page}
                            max={totalPages} min={1}
                        onChange={(e) => setPage(Number(e.target.value))} />
                    </PopoverContent>
                </Popover>
                <Button onClick={() => setPage(page + 1)} disabled={page >= totalPages}>
                    다음
                </Button>
            </div>
        </div>
    )
}

export default TagsPage;